# coding=utf-8

"""Firebase plugin window class."""

import os
import time
import subprocess

from PyQt5.QtGui import QDesktopServices
from PyQt5.QtCore import Qt, QUrl
from PyQt5.QtWidgets import QAction, QToolButton, QHBoxLayout, QMessageBox, QMenu, QFileDialog, QWidgetAction, QLabel

from app.utils import tmpPath
from app.plugins.core.plugin import IdePlugin
from app.plugins.editor.plugin import CodeEditorPlugin
from app.framework.QHelper import QHelper
from app.framework.plugin_system import PluginWindow, PluginWindowType

from firebase_plugin.build import BuildFirebaseThread
from firebase_plugin.progress import FireBaseWidget
from firebase_plugin.board import ChoosingWidget
from firebase_plugin.about_headspin_dialog import AboutDialog

from app.params import ROOT
from app.utils import get_platform


class FireBaseWindow(PluginWindow):
    """Firebase插件窗口."""

    PROPERTIES = {
        "windowType": PluginWindowType.WT_DOCK,  # docking属性
        "dockingTitle": "Firebase Window",
        "dockingVisible": False,
        "dockingArea": Qt.AllDockWidgetAreas,
        "dockingPosition": Qt.LeftDockWidgetArea,
    }

    def __init__(self):
        super(FireBaseWindow, self).__init__()
        self.build_package_thread = None
        self.isPackaging = False
        self.initUi()
        self.edit_que = []
        self.multi_build = False

    def createMenu(self, menuBar):
        """创建专属下拉菜单."""
        self.setupTranslateActionText()

        self.firebase_menu = QMenu(self.menu_name)
        self.firebase_resign_action = QAction(self.build_apk_str, self, triggered=self.handle_resign)
        self.firebase_choose_action = QAction(self.build_multiple_str, self, triggered=self.handle_multi_build)

        # about partners:
        if get_platform() == "Darwin":
            self.label = QLabel("")
            self.partners_label_action = QAction(self.partner_str, self)
            self.partners_label_action.setDisabled(True)
        else:
            self.label = QLabel(self.partner_str)
            self.partners_label_action = QWidgetAction(self)
            self.partners_label_action.setDefaultWidget(self.label)

        # self.about_firebase_action = QAction(self.tr("About Firebase"), self, triggered=self.onAboutFirebase)
        self.about_headspin_action = QAction(self.about_headspin_str, self, triggered=self.onAboutHeadspin)
        self.about_headspin_action.setMenuRole(QAction.NoRole)

        # 组件UI结构
        self.firebase_menu.addAction(self.firebase_resign_action)
        self.firebase_menu.addAction(self.firebase_choose_action)
        self.firebase_menu.addSeparator()
        self.firebase_menu.addAction(self.partners_label_action)
        # self.firebase_menu.addAction(self.about_firebase_action)
        self.firebase_menu.addAction(self.about_headspin_action)

        menuBar.addMenu(self.firebase_menu)

    def setupTranslateActionText(self):
        """初始化actions的文字."""
        if IdePlugin.conf.get("preferLocale", None) == "zh_CN":
            self.build_apk_str = "打包Instrumentation测试Apk"
            self.build_multiple_str = "打包多个脚本"
            self.menu_name = "海外设备"
            self.partner_str = " 金牌伙伴:"
            self.about_headspin_str = "关于Headspin"
        else:
            self.build_apk_str = self.tr("Build Instrumentation Test Apk")
            self.build_multiple_str = self.tr("Build Multiple Scripts")
            self.menu_name = self.tr("Overseas Devices")
            self.partner_str = self.tr(" Gold Partners:")
            self.about_headspin_str = self.tr("About HeadSpin")

    def translateActions(self):
        """翻译actions."""
        self.setupTranslateActionText()
        self.firebase_resign_action.setText(self.build_apk_str)
        self.firebase_choose_action.setText(self.build_multiple_str)
        self.about_headspin_action.setText(self.about_headspin_str)
        self.label.setText(self.partner_str)
        self.firebase_menu.setTitle(self.menu_name)

    def initUi(self):
        # return
        self.setObjectName("FireBaseWindow")
        self.resize(150, 300)

        self.main_layout = QHBoxLayout(self)

        # self.firebase_compress_action = QAction("Compress")
        # self.firebase_compress_action.setToolTip("Compress Script")
        # self.firebase_compress_button = QToolButton(self)
        # self.firebase_compress_button.setDefaultAction(self.firebase_compress_action)
        # self.firebase_compress_button.clicked.connect(self.handle_compress)

        self.firebase_resign_action = QAction("Resign Firebase Package")
        self.firebase_resign_action.setToolTip("Resign Script")
        self.firebase_resign_button = QToolButton(self)
        self.firebase_resign_button.setDefaultAction(self.firebase_resign_action)
        self.firebase_resign_button.clicked.connect(self.handle_resign)
        self.main_layout.addWidget(self.firebase_resign_button)

    def onAboutHeadspin(self):
        """点击关于HeadSpin."""
        about_headspin_dialog = AboutDialog()
        about_headspin_dialog.exec_()

    def onAboutFirebase(self):
        """点击关于Firebase."""
        print("about firebase")  # 暂时不做Firebase的about弹窗

    def detect_java(self):
        try:
            proc = subprocess.Popen(['jarsigner'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            proc.wait()
            for line in proc.stdout:
                if "No Java" in str(line):
                    return False
        except Exception:
            return False

        return True

    def openDoc(self):
        if IdePlugin.conf.get("preferLocale", None) == "zh_CN":
            index_html = "http://airtest.netease.com/docs/cn/8_plugins/1_firebase.html"
        else:
            index_html = "http://airtest.netease.com/docs/en/1_online_help/firebase_building.html"
        QDesktopServices.openUrl(QUrl(index_html))

    def openPath(self, path):
        try:
            # startfile只有windows支持
            path = os.path.join(os.getcwd(), path)
            os.startfile(path)
        except:
            import subprocess
            subprocess.call(["open", path])
        self.finish_build()

    def finish_build(self):
        print("finish it")
        # reply = QMessageBox.question(self, "", "Apk Generated! Do you want to open apk path?", \
        #                              QMessageBox.Yes, QMessageBox.No)
        # if reply == QMessageBox.Yes:
        #     path = os.path.join(ROOT, "temp", "apk")
        #     self.openPath(path)
        self.isPackaging = False
        self.build_package_thread.set_close()
        IdePlugin.MainWindow.statusbarShowTip("")
        self.progress.close()

    def progress_complete(self):
        IdePlugin.MainWindow.statusbarShowTip("")
        self.progress.open_button.setDisabled(False)

    def script_not_exist(self):
        QMessageBox.warning(self, "Warning", "Edit and save script first!")
        self.isPackaging = False
        IdePlugin.MainWindow.statusbarShowTip("")
        self.progress.close()

    def handle_open(self):
        path = tmpPath("apk")
        self.openPath(path)

    def handle_pause(self):
        if self.build_package_thread.resign_process:
            print("---------------kill process")
            self.build_package_thread.resign_process.kill()
        self.build_package_thread.quit()
        self.finish_build()

    def handle_choose_script(self, num):
        source_dir = CodeEditorPlugin().editor_tab.getCurrentProjectPath()
        file_name = QFileDialog.getOpenFileName(self, "Choose Script", source_dir, "All Files (*);;Python Files (*.py)")
        for key, item in enumerate(self.edit_que):
            if item.text() == os.path.split(file_name[0])[0]:
                self.edit_que[key].setText("")

        path_split = os.path.split(file_name[0])
        if os.path.splitext(path_split[0])[1] == '.air':
            self.edit_que[num].setText(path_split[0])

    def handle_add_item(self):
        self.choosing_board.add_item()

    def handle_multi_build_script(self):
        self.multi_build = True
        self.build_list = []
        for item in self.edit_que:
            if item.text() != "":
                self.build_list.append(item.text())
        self.handle_resign()
        self.multi_build = False

    def handle_multi_build(self):
        self.choosing_board = ChoosingWidget(firebase_window=self)
        # self.choosing_board.choose_file_button.clicked.connect(self.handle_choose_script)
        self.choosing_board.setGeometry(400, 400, 380, 70)
        self.choosing_board.add_button.clicked.connect(self.handle_add_item)
        self.choosing_board.build_button.clicked.connect(self.handle_multi_build_script)
        self.choosing_board.show()

    def handle_resign(self):
        if not self.detect_java():
            self.openDoc()
            return

        if self.isPackaging:
            return
        else:
            self.isPackaging = True
        IdePlugin.MainWindow.statusbarShowTip("Building firebase apk...")

        self.progress = FireBaseWidget(firebase_window=self)
        self.progress.setObjectName("Building progress...")
        self.progress.setGeometry(400, 400, 380, 70)
        self.progress.open_button.clicked.connect(self.handle_open)
        self.progress.close_button.clicked.connect(self.handle_pause)
        self.progress.open_button.setDisabled(True)
        self.progress.show()

        while True:
            if self.build_package_thread and not self.build_package_thread.isFinished():
                time.sleep(0.3)
            else:
                break

        if hasattr(self, "choosing_board"):
            self.choosing_board.close()
        if self.multi_build:
            self.build_package_thread = BuildFirebaseThread(self.build_list)
        else:
            self.build_package_thread = BuildFirebaseThread([CodeEditorPlugin().editor_tab.getCurrentProjectPath()])
        self.build_package_thread.start()
        self.build_package_thread.SIGNAL_BUILD_COMPLETE.connect(self.progress_complete)
        self.build_package_thread.SIGNAL_SCRIPT_NOT_EXIST.connect(self.script_not_exist)
        self.build_package_thread.SIGNAL_UPDATE_PROGRESS.connect(self.progress.update)
