#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import time
from subprocess import Popen, STDOUT

from PyQt5.QtGui import QIcon
from PyQt5.QtCore import Qt, pyqtSignal, QSize, QCoreApplication
from PyQt5.QtWidgets import QAction, QMessageBox

from airtest import aircv

from app import ga
from app.params import WIN_OS
from app.utils import loadStyleSheet, tmpPath, get_platform
from app.framework.QHelper import QHelper
from app.framework.plugin_system import PluginWindow, PluginWindowType
from app.plugins.core.plugin import IdePlugin
from app.plugins.editor.plugin import CodeEditorPlugin as EditorPlugin
from app.plugins.airtest.airtest_assistant import RecordAssistant
from app.plugins.airtest.airtest_recorder import ManualRecorder

from selenium_plugin.airtest_chrome import SeleniumThread
from selenium_plugin.selenium_assistant import SeleniumAssistant
from selenium_plugin.gui.ui_selenium_window import Ui_SeleniumWindow

if get_platform() == WIN_OS:
    from app.plugins.device.win.win_client import WinClient

class SeleniumWindow(PluginWindow, Ui_SeleniumWindow):

    PROPERTIES = {
        "windowType": PluginWindowType.WT_DOCK,
        # docking属性
        "dockingTitle": "Selenium Window",
        "dockingVisible": True,
        "dockingArea": Qt.AllDockWidgetAreas,
        "dockingPosition": Qt.LeftDockWidgetArea,
    }

    SIGNAL_START_RECORD = pyqtSignal(bool)
    SIGNAL_START_TEMPLATE = pyqtSignal(bool)
    SIGNAL_INSERT_PLAIN_CODE = pyqtSignal(str)
    SIGNAL_ENABLE_BUTTONS = pyqtSignal(bool, str)
    SIGNAL_START_CHROME = pyqtSignal(bool, str)

    def __init__(self):
        super(SeleniumWindow, self).__init__()
        self.setupUi(self)
        self.sele_assistant = SeleniumAssistant(self)
        self.build_package_thread = None
        self.selenium_thread = None
        self.isPackaging = False
        self.edit_que = []
        self.multi_build = False
        self.createActions()
        self.classifyButtons()
        self.createSlots()
        self.slot_signal()

    def slot_signal(self):
        self.sele_assistant.SIGNAL_EDIT_CODE.connect(EditorPlugin().insertCode)
        self.sele_assistant.SIGNAL_START_RECORDING.connect(EditorPlugin().confirmWebCode)
        self.sele_assistant.SIGNAL_RESET_LOGVIEW.connect(self.reset_log)

    def classifyButtons(self):
        self.all_buttons = [self.newTabBtn, self.backBtn, self.snapshotBtn, self.textBtn,
                            self.forwardBtn, self.touchBtn, self.selenium_template_toolbtn,
                            self.startWebBtn, self.assertBtn, self.previousTabBtn]
        self.recording_buttons = [self.snapshotBtn, self.previousTabBtn, self.backBtn,
                                  self.forwardBtn, self.startWebBtn, self.newTabBtn]

    def createSlots(self):
        # self.selenium_auto_record_toolbtn.clicked.connect(self.recordSlot)
        self.start_chrome_btn.clicked.connect(self.start_chrome)
        self.textBtn.clicked.connect(self.textSlot)
        self.snapshotBtn.clicked.connect(self.snapshotSlot)
        # self.sleepBtn.clicked.connect(self.sleepSlot)
        self.backBtn.clicked.connect(self.backSlot)
        self.touchBtn.clicked.connect(self.touchSlot)
        self.newTabBtn.clicked.connect(self.switchToLatestSlot)
        self.previousTabBtn.clicked.connect(self.switchToLastSlot)
        self.forwardBtn.clicked.connect(self.forwardSlot)
        self.startWebBtn.clicked.connect(self.startWebSlot)
        self.assertBtn.clicked.connect(self.assertSlot)
        self.assertTemplateBtn.clicked.connect(self.assertTemplateSlot)
        self.airtestTouchBtn.clicked.connect(self.airtestTouchSlot)

    def createActions(self):
        self.record_action = QAction(self.tr("Record"))
        self.record_action.setCheckable(True)
        self.record_action.toggled.connect(self.SIGNAL_START_RECORD)

        self.selenium_auto_record_toolbtn.setDefaultAction(self.record_action)
        self.selenium_auto_record_toolbtn.setIconSize(QSize(18, 18))

        self.selenium_template_toolbtn.setIconSize(QSize(18, 18))
        self.selenium_template_toolbtn.setCheckable(True)
        self.selenium_template_toolbtn.toggled.connect(self.SIGNAL_START_TEMPLATE)

        # self.start_chrome_toolbtn.setIconSize(QSize(18, 18))
        # self.selenium_template_toolbtn.setCheckable(True)
        # self.selenium_template_toolbtn.toggled.connect(self.SIGNAL_START_CHROME)

        # 为所有的actions注册图标：
        self.registerIconImage()

    def registerIconImage(self):
        """注册图片资源."""
        self.touchBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/touch_n.svg", ":/images/selenium_window/moonlight/touch_n.svg", ":/images/selenium_window/moonlight/touch_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/touch_n.png", ":/images/selenium_window/darkshadow/touch_n.png", ":/images/selenium_window/darkshadow/touch_d.png"]
        }
        self.airtestTouchBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/airtest_touch_n.svg", ":/images/selenium_window/moonlight/airtest_touch_n.svg", ":/images/selenium_window/moonlight/airtest_touch_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/airtest_touch_n.png", ":/images/selenium_window/darkshadow/airtest_touch_n.png", ":/images/selenium_window/darkshadow/airtest_touch_d.png"]
        }
        self.startChromeBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/browser_n.svg", ":/images/selenium_window/moonlight/browser_n.svg", ":/images/selenium_window/moonlight/browser_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/browser_n.png", ":/images/selenium_window/darkshadow/browser_n.png", ":/images/selenium_window/darkshadow/browser_n.png"]
        }
        self.startWebBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/start_web_n.svg", ":/images/selenium_window/moonlight/start_web_n.svg", ":/images/selenium_window/moonlight/start_web_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/start_web_n.png", ":/images/selenium_window/darkshadow/start_web_n.png", ":/images/selenium_window/darkshadow/start_web_n.png"]
        }
        self.textBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/text_n.svg", ":/images/selenium_window/moonlight/text_n.svg", ":/images/selenium_window/moonlight/text_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/text_n.png", ":/images/selenium_window/darkshadow/text_n.png", ":/images/selenium_window/darkshadow/text_n.png"]
        }
        self.assertBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/assert_n.svg", ":/images/selenium_window/moonlight/assert_n.svg", ":/images/selenium_window/moonlight/assert_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/assert_n.png", ":/images/selenium_window/darkshadow/assert_n.png", ":/images/selenium_window/darkshadow/assert_n.png"]
        }
        self.assertTemplateBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/assert_airtest_n.svg", ":/images/selenium_window/moonlight/assert_airtest_n.svg", ":/images/selenium_window/moonlight/assert_airtest_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/assert_airtest_n.png", ":/images/selenium_window/darkshadow/assert_airtest_n.png", ":/images/selenium_window/darkshadow/assert_airtest_n.png"]
        }
        self.switchLatestBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/switch_to_latest_n.svg", ":/images/selenium_window/moonlight/switch_to_latest_n.svg", ":/images/selenium_window/moonlight/switch_to_latest_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/switch_to_latest_n.png", ":/images/selenium_window/darkshadow/switch_to_latest_n.png", ":/images/selenium_window/darkshadow/switch_to_latest_n.png"]
        }
        self.switchLastBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/switch_to_last_n.svg", ":/images/selenium_window/moonlight/switch_to_last_n.svg", ":/images/selenium_window/moonlight/switch_to_last_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/switch_to_last_n.png", ":/images/selenium_window/darkshadow/switch_to_last_n.png", ":/images/selenium_window/darkshadow/switch_to_last_n.png"]
        }
        self.backBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/back_n.svg", ":/images/selenium_window/moonlight/back_n.svg", ":/images/selenium_window/moonlight/back_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/back_n.png", ":/images/selenium_window/darkshadow/back_n.png", ":/images/selenium_window/darkshadow/back_n.png"]
        }
        self.forwardBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/forward_n.svg", ":/images/selenium_window/moonlight/forward_n.svg", ":/images/selenium_window/moonlight/forward_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/forward_n.png", ":/images/selenium_window/darkshadow/forward_n.png", ":/images/selenium_window/darkshadow/forward_n.png"]
        }
        self.snapshotBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/snapshot_n.svg", ":/images/selenium_window/moonlight/snapshot_n.svg", ":/images/selenium_window/moonlight/snapshot_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/snapshot_n.png", ":/images/selenium_window/darkshadow/snapshot_n.png", ":/images/selenium_window/darkshadow/snapshot_n.png"]
        }
        self.recordBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/auto_record_n.svg", ":/images/selenium_window/moonlight/auto_record_n.svg", ":/images/selenium_window/moonlight/auto_record_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/auto_record_n.png"]
        }
        self.seleniumTemplateBtnImgDict = {
            "MoonLight": [":/images/selenium_window/moonlight/select_n.svg", ":/images/selenium_window/moonlight/select_n.svg", ":/images/selenium_window/moonlight/select_d.svg"],
            "DarkShadow": [":/images/selenium_window/darkshadow/select_n.png"]
        }

        # 注册图片资源：(插件的需要额外使用Qhelper注册)
        QHelper.registerAndSetIconImage(self.touchBtn, self.touchBtnImgDict)
        QHelper.registerAndSetIconImage(self.airtestTouchBtn, self.airtestTouchBtnImgDict)
        QHelper.registerAndSetIconImage(self.start_chrome_btn, self.startChromeBtnImgDict)
        QHelper.registerAndSetIconImage(self.startWebBtn, self.startWebBtnImgDict)
        QHelper.registerAndSetIconImage(self.textBtn, self.textBtnImgDict)
        QHelper.registerAndSetIconImage(self.assertBtn, self.assertBtnImgDict)
        QHelper.registerAndSetIconImage(self.assertTemplateBtn, self.assertTemplateBtnImgDict)
        QHelper.registerAndSetIconImage(self.newTabBtn, self.switchLatestBtnImgDict)
        QHelper.registerAndSetIconImage(self.previousTabBtn, self.switchLastBtnImgDict)
        QHelper.registerAndSetIconImage(self.backBtn, self.backBtnImgDict)
        QHelper.registerAndSetIconImage(self.forwardBtn, self.forwardBtnImgDict)
        QHelper.registerAndSetIconImage(self.snapshotBtn, self.snapshotBtnImgDict)
        QHelper.registerAndSetIconImage(self.record_action, self.recordBtnImgDict)
        QHelper.registerAndSetIconImage(self.selenium_template_toolbtn, self.seleniumTemplateBtnImgDict)

    def setMoonlightStyle(self):
        """每个控件载入图片资源,并载入qss资源."""
        loadStyleSheet(self, ':/qss/moonlight/airtest_window.qss')

    def setDarkshadowStyle(self):
        loadStyleSheet(self, ':/qss/darkshadow/airtest_window.qss')

    def reset_log(self, content=None):
        pyeditor_plugin = EditorPlugin()
        pyeditor_plugin.clearLogView()
        if content is not None:
            pyeditor_plugin.insertLogView(content)

    def startWebSlot(self):
        """selenium 开启一个网站"""
        default_addr = "Write your test web address!"
        # print self.sele_assistant.web_addr
        web_addr = self.sele_assistant.get_web_addr() or default_addr
        _code = "driver.get(\"{web_addr}\")".format(web_addr=web_addr)
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        self.startWebBtn.setChecked(False)

    def switchToLatestSlot(self):
        """selenium 切换标签"""
        _code = "driver.switch_to_new_tab()"
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        self.newTabBtn.setChecked(False)

    def switchToLastSlot(self):
        _code = "driver.switch_to_previous_tab()"
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        self.previousTabBtn.setChecked(False)

    def closeSlot(self):
        _code = "driver.close()"
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        # self.switchBtn.setChecked(False)

    def assertSlot(self):
        """selenium assert事件"""
        if not self.assertBtn.isChecked():
            self.stop_inspect(self.assertBtn)
        else:
            self.set_all_btn(False, self.assertBtn)
            self.start_inspect("assert")

    def assertTemplateSlot(self):
        if get_platform() != WIN_OS:
            QMessageBox.warning(None, self.tr("Warning"), self.tr("Current OS is not 'Windows'! You can't use airtest function of Airtest-Selenium. > <"))
            self.assertTemplateBtn.setChecked(False)
            return False
        code = self.get_capture_code_by_func("driver.assert_template")
        self.SIGNAL_INSERT_PLAIN_CODE.emit(code)
        self.assertTemplateBtn.setChecked(False)

    def airtestTouchSlot(self):
        if get_platform() != WIN_OS:
            QMessageBox.warning(None, self.tr("Warning"), self.tr("Current OS is not 'Windows'! You can't use airtest function of Airtest-Selenium. > <"))
            self.airtestTouchBtn.setChecked(False)
            return False
        code = self.get_capture_code_by_func("driver.airtest_touch")
        self.SIGNAL_INSERT_PLAIN_CODE.emit(code)
        self.airtestTouchBtn.setChecked(False)

    def touchSlot(self):
        """selenium touch事件"""
        if not self.touchBtn.isChecked():
            self.stop_inspect(self.touchBtn)
        else:
            self.set_all_btn(False, self.touchBtn)
            self.start_inspect("click")

    def textSlot(self):
        """selenium 输入文本"""
        if not self.textBtn.isChecked():
            self.stop_inspect(self.textBtn)
        else:
            self.set_all_btn(False, self.textBtn)
            self.start_inspect("text")

    def snapshotSlot(self):
        """selenium 截图"""
        _code = "driver.snapshot()"
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        self.snapshotBtn.setChecked(False)

    def backSlot(self):
        """selenium 返回上层页面"""
        _code = "driver.back()"
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        self.backBtn.setChecked(False)

    def forwardSlot(self):
        """selenium 返回上层页面"""
        _code = "driver.forward()"
        self.SIGNAL_INSERT_PLAIN_CODE.emit(_code)
        self.forwardBtn.setChecked(False)

    def get_capture_code_by_func(self, func):
        ret, screenshot, midpos, vector = WinClient().doCapture("airtest_touch")
        code =""
        if ret is not None and screenshot is not None:
            fname = ManualRecorder().saveImage(ret, screenshot)
            img = aircv.utils.cv2_2_pil(ret)
            funcname = func
            r_click = False
            resolution = WinClient().getCurrentDeviceResolution()
            print("device resolution: ", resolution)
            code = RecordAssistant.generateRecordPyCode(funcname, fname, r_click, vector, None, midpos, resolution)

        return code

    def set_all_btn(self, status, exclude_btn=None):
        """同时设置所有button"""
        for btn in self.all_buttons:
            if exclude_btn and btn == exclude_btn:
                continue
            btn.setChecked(status)

    def set_enable_btn(self, enable_btns):
        for btn in self.all_buttons:
            if btn not in enable_btns:
                btn.setDisabled(True)
            else:
                btn.setDisabled(False)

    def get_tmp_dir(self, child_path=""):
        return tmpPath(child_path)

    def start_chrome(self):
        self.sele_assistant.init_code()
        chrome_path = IdePlugin.conf.get("ChromePath", "")
        if chrome_path == "":
            QMessageBox.warning(self, "warning", self.tr("Please set Chrome path on Options!"))
            return

        try:
            current_path = os.getcwd()
            os.chdir(os.path.dirname(chrome_path))
            tmp_dir = self.get_tmp_dir("selenium_user_data")
            param_user_data = '--user-data-dir={user_dir}'.format(user_dir=tmp_dir)
            if "darwin" in sys.platform:
                cmd = ['/Applications/Google Chrome.app/Contents/MacOS/Google Chrome',
                       '--remote-debugging-port=9222', param_user_data]
            else:
                cmd = ['chrome.exe', '--remote-debugging-port=9222', param_user_data]
            print(cmd)
            Popen(cmd, stderr=STDOUT)
            os.chdir(current_path)
            ga.event("selenium", "start_chrome")
        except Exception:
            import traceback
            print(traceback.format_exc())
        time.sleep(1.5)

    def start_recording(self):
        if self.sele_assistant.is_inspecting:
            self.stop_inspect()
        self.set_enable_btn(self.recording_buttons)
        self.selenium_thread = SeleniumThread(self.sele_assistant)
        self.selenium_thread.start()

    def stop_recording(self):
        if hasattr(self, "selenium_thread"):
            self.selenium_thread.finish_recording()
        self.set_enable_btn(self.all_buttons)

    def start_inspect(self, _type):
        if self.sele_assistant.is_inspecting:
            print("change inspect type...", _type)
            self.sele_assistant.change_operation(_type)
        else:
            self.sele_assistant.start_inspect("normal", _type)

    def stop_inspect(self, btn=None):
        self.sele_assistant.stop_inspect(btn)

    def customRetranslateUi(self):
        """切换语言版本时需要加载的属性."""
        pass
